from contextlib import contextmanager
from typing import Any, Callable, Dict, List, Tuple

from rabbitmq_alphamoon import IQueueConnector


class ConnectorMock(IQueueConnector):
    def __init__(self, *args, **kwargs):
        self.__opened_connection = False
        self.init_args = args
        self.init_kwargs = kwargs

    @contextmanager
    def open_connection(self):
        if self.__opened_connection:
            raise RuntimeError("Connection already opened")
        try:
            self.__opened_connection = True
            yield self
        finally:
            self.__opened_connection = False

    def _verify_connection_opened(self):
        if not self.__opened_connection:
            raise RuntimeError("Connection not opened")

    def consume_forever(self, process_message_callback: Callable[[Any, Dict[str, Any]], None]):
        raise NotImplementedError("consume_forever not implemented in " + type(self).__name__)

    def publish(self, message: Any) -> None:
        raise NotImplementedError("publish not implemented in " + type(self).__name__)

    @property
    def message_count(self) -> int:
        raise NotImplementedError("message_count not implemented in " + type(self).__name__)


def create_consumer_connector_mock(incoming_messages: List[Tuple[Any, Dict[str, Any]]]) -> object:
    class ConsumerConnectorMock(ConnectorMock):
        def consume_forever(self, process_message_callback: Callable[[Any, Dict[str, Any]], None]):
            self._verify_connection_opened()
            for message, info in incoming_messages:
                process_message_callback(message, info)

    return ConsumerConnectorMock



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path

from setuptools import setup

import rabbitmq_alphamoon


def read(subpath):
    return (Path(__file__).parent / subpath).read_text(encoding='utf-8')


setup(
    name='rabbitmq-alphamoon',
    version=rabbitmq_alphamoon.__version__,
    author='Piotr Dąbrowski',
    author_email='piotr.dabrowski@alphamoon.ai',
    maintainer='Piotr Dąbrowski',
    maintainer_email='dev@alphamoon.ai',
    license='MIT',
    url='https://gitlab.com/alphamoon/internal_tools/rabbitmq-alphamoon',
    description='pika-based RabbitMQ connector with built in JSON serialization/deserialization',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    py_modules=['rabbitmq_alphamoon'],
    python_requires='>=3.8',
    install_requires=[
        'pika==1.2.0',
    ],
    packages=['rabbitmq_alphamoon'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
    ],
)
